@@ -1,7 +1,6 @@
-#' @template dbispec
 #' @format NULL
 spec_connection <- c(
-  spec_connection_connect,
+  spec_connection_disconnect,
   spec_connection_data_type,
   spec_connection_get_info
 )
